@@ -10,8 +10,7 @@
 #      getExtraVariables
 #      getCoxphControl
 #      checkSubcohortStrataNotEmpty
-#   classRelatedFunctions.R
-#      setS3class
+#   cchs-class.R
 #      print.cchs
 #      summary.cchs
 #   tiedEventTimeSubroutines.R
@@ -31,25 +30,32 @@
 #      minDifference
 #      hasUnusedLevels
 #      removeNAsFromVector
+#      setS3class
 
 # R style:
 # - boolean variables (e.g. isCase, inSubcohort) should be logicals (FALSE/
 #   TRUE), not 0/1.  
 # - to get a column called b from a data-frame called a, use a$b instead of 
 #   a[["b"]], and use $ or [[ in preference to a[,b] 
-# - stop("INTERNAL ERROR: ...") means something is wrong with the code, not with 
-#   what the user did
+# - stop("INTERNAL ERROR: ...") means something is wrong with the code, not  
+#   with what the user did
 
-###############################################################################
+# been renamed. I think these are here because I accidentally deleted 
+# borganIII folder and then recreated it by copying from C:\MyDocs_backupOnC, 
+# and that contains excess files due to having been created using xcopy. 
+
+################################################################################
 # Main function to calculate Borgan's Estimator III. 
 
 cchs <- function(formula, data=sys.parent(), inSubcohort, stratum, 
 			samplingFractions, cohortStratumSizes, precision=NULL, 
 			swap=TRUE, dropNeverAtRiskRows=TRUE, dropSubcohEventsDfbeta=FALSE, 
 			adjustSampFracIfAnyNAs=FALSE, keepAllCoxphElements=FALSE, 
-			verbose=FALSE, coxphControl, ...) {
+			confidenceLevel=0.95, verbose=FALSE, annotateErrors=TRUE, 
+			coxphControl, ...) {
+	# Note: the "0.95" is hard-coded in cchs-class.R and cchs.Rd as well. 
 	
-	#############################################################################
+	############################################################################
 	# Deal with some of the arguments, etc.  
 	originalCall <- match.call() 
 
@@ -66,30 +72,34 @@ cchs <- function(formula, data=sys.parent(), inSubcohort, stratum,
 	
 	# Check several of the arguments, and make inSubcohort be boolean/logical. 
 	# (Checking class(formula) must be done before identifying NAs.)
-	checkCchsArguments(formula, data, inSubcohort, stratum) 
+	checkCchsArguments(formula, data, inSubcohort, stratum, confidenceLevel) 
 	inSubcohort <- as.logical(inSubcohort) 
 	
-	#############################################################################
+	############################################################################
 	# Find all rows that have NAs in the columns of data that appear in formula, 
 	# including the event/censoring variable. (This throws an error in certain 
-	# situations, so add appropriate extra text to those error-messages.) See 
-	# also extra comment 1 below.
-	tryCatch(
-		modelFrame <- model.frame(formula, data, na.action=na.omit),
-		error=function(e) {
-			if (e$call[[1]] == "Surv") {
-				errorReason <- "the Surv object being illegal"
-			} else if (grepl("^object .* not found$", e$message)) {
-				errorReason <- "a variable in the model-formula\nnot existing"
-			} else if (grepl("^variable lengths differ", e$message)) {
-				errorReason <- 
+	# situations, so add appropriate extra text to those error-messages, if 
+	# annotateErrors=TRUE.) See also extra comment 1 below.
+	if (annotateErrors) {
+		tryCatch(
+			modelFrame <- model.frame(formula, data, na.action=na.omit),
+			error=function(e) {
+				if (e$call[[1]] == "Surv") {
+					errorReason <- "the Surv object being illegal"
+				} else if (grepl("^object .* not found$", e$message)) {
+					errorReason <- "a variable in the model-formula\nnot existing"
+				} else if (grepl("^variable lengths differ", e$message)) {
+					errorReason <- 
 						"variables in the model-formula having\ndifferent lengths"
-			} else {
-				errorReason <- "a problem with the model-formula"
+				} else {
+					errorReason <- "a problem with the model-formula"
+				}
+				stop(e$message, "\n[This error is caused by ", errorReason, ".]")
 			}
-			stop(e$message, "\n[This error is caused by ", errorReason, ".]")
-		}
-	)
+		)
+	} else {
+		modelFrame <- model.frame(formula, data, na.action=na.omit)
+	}
 	rowsToDrop <- attr(modelFrame, "na.action")
 	if (is.null(rowsToDrop)) {
 		rowsToDrop <- numeric(0)
@@ -113,16 +123,16 @@ cchs <- function(formula, data=sys.parent(), inSubcohort, stratum,
 	# Make stratum a factor. 
 	if (!is.factor(stratum)) {
 		stratum <- tryCatch(
-				as.factor(stratum),
-				error=function(e) stop("stratum cannot be converted to a factor")
+			as.factor(stratum),
+			error=function(e) stop("stratum cannot be converted to a factor")
 		)
 	}
 	
 	# Drop any unused levels from stratum. See extra comment 2 below. 
 	stratum <- droplevels(stratum) 
 
-	# Check that all strata have at least one row in the subcohort. This will be 
-	# assumed by checkCohortStratumSizes. 
+	# Check that all strata have at least one row in the subcohort. This will  
+	# be assumed by checkCohortStratumSizes. 
 	checkSubcohortStrataNotEmpty(stratum, inSubcohort,
 		message="before dropping NA-containing rows")
 
@@ -135,7 +145,7 @@ cchs <- function(formula, data=sys.parent(), inSubcohort, stratum,
 	#	print(head(data))
 	#}
 
-	#############################################################################
+	############################################################################
 	# Deal with the sampling fractions. 
 
 	# Check samplingFractions or cohortStratumSizes, whichever was given. 
@@ -155,13 +165,13 @@ cchs <- function(formula, data=sys.parent(), inSubcohort, stratum,
 		samplingFractions <- adjustSamplingFractions(samplingFractions, 
 				stratum, inSubcohort, rowsToKeep, verbose)  
 	
-	#############################################################################
+	############################################################################
 	# Remake modelFrame, which contains the response and the model matrix, and 
 	# drop the NA-containing rows from it. 
 	modelFrame <- model.frame(formula, data, na.action=na.pass)
 	modelFrame <- modelFrame[rowsToKeep,]
 	# The previous line drops the appropriate rows from all attributes of 
-	# modelFrame, so the lines with model.matrix and model.response will work OK.
+	# modelFrame, so the lines with model.matrix and model.response will work.
 	if (nrow(modelFrame) == 0) stop("no observations")
 	
 	# Drop the appropriate rows from other variables. 
@@ -177,24 +187,24 @@ cchs <- function(formula, data=sys.parent(), inSubcohort, stratum,
 			message="after dropping NA-containing rows")
 	
 	# Make the model-matrix from the model-frame. 
-	modelMatrix <- model.matrix(attr(modelFrame,"terms"), modelFrame)
+	modelMatrix <- model.matrix(attr(modelFrame, "terms"), modelFrame)
 	modelMatrix <- modelMatrix[, -1, drop=FALSE]  # remove "(Intercept)" column 
 	# (the "-1" is probably safer than explicitly using the column's name) 
 	
 	# Deal with special terms in the model-formula. "cluster" and "strata" are 
-	# not allowed but "offset" is. (coxph also has "tt", which is similar but is 
-	# passed as an argument to coxph, not in the model-formula.) 
+	# not allowed but "offset" is. (coxph also has "tt", which is similar but  
+	# is passed as an argument to coxph, not in the model-formula.) 
 	prohibitedSpecialNames <-  c("cluster", "strata")
 	prohibitedSpecialTermsList <- 
 			attr(terms(formula, specials=prohibitedSpecialNames), "specials")  
 			# "terms(modelFrame, specials=prohibitedSpecialNames)" does not work
 	for (i in prohibitedSpecialNames)
 		if (!is.null(prohibitedSpecialTermsList[[i]]))
-			stop("Model-formula is not allowed to contain these special terms: ", 
+			stop("formula is not allowed to contain these special terms: ", 
 					paste(prohibitedSpecialNames, collapse=", "))
 
-	#############################################################################
-	# Get the Surv object and from that get the entry-time, exit-time, and event/
+	############################################################################
+	# Get the Surv object and from that get entry-time, exit-time, and event/
 	# censoring variable. (Surv coerces the event/censoring variable to 0/1.) 
 	survObject <- model.response(modelFrame)
 	if (!inherits(survObject, "Surv")) 
@@ -215,7 +225,12 @@ cchs <- function(formula, data=sys.parent(), inSubcohort, stratum,
 		timeAtExit <- survObject[,"stop"]
 		isCase <- survObject[,"status"]
 	}
-
+	# Note on Surv types: "counting" is intended for the scenario where each 
+	# patient has multiple rows, one for each time-interval. But it is also 
+	# suitable for the scenario where each patient has a non-zero entry-time as 
+	# well as the exit-time (the exit-time is when they are censored or have 
+	# the event). The former scenario is "duration as timescale" and the latter 
+	# scenario is "age as timescale". 
 	# Names of corresponding variables in cch: 
 	# cens=isCase, texit=timeAtExit, tentry=timeAtEntry, X=modelMatrix
 	
@@ -223,7 +238,8 @@ cchs <- function(formula, data=sys.parent(), inSubcohort, stratum,
 	isCase <- as.logical(isCase) 
 	isNonsubcohortNoncase <- !inSubcohort & !isCase
 	if (any(isNonsubcohortNoncase)) 
-		stop("there are ", sum(isNonsubcohortNoncase), " non-subcohort non-cases")
+		stop("there are ", sum(isNonsubcohortNoncase), 
+				" non-subcohort non-cases")
 
 	# Store numbers of cases etc., which will be put in the final result object. 
 	n <- nrow(modelMatrix)
@@ -233,7 +249,7 @@ cchs <- function(formula, data=sys.parent(), inSubcohort, stratum,
 			nonsubcohortCases=sum(!inSubcohort)
 	)
 	
-	#############################################################################
+	############################################################################
 	# Deal with tied event-times (and check all times using precision).
 	fixedTimes <- fixTiedEventTimes(timeAtEntry, timeAtExit, isCase, precision)
 	timeAtEntry <- fixedTimes$timeAtEntry
@@ -246,7 +262,9 @@ cchs <- function(formula, data=sys.parent(), inSubcohort, stratum,
 	entryAndExitTimes <- c(timeAtEntry, timeAtExit)
 	epsilon <- minDifference(entryAndExitTimes) / 10
 	if (verbose) cat("## epsilon=", epsilon, "\n", sep="")
-	if (epsilon == 0) stop("epsilon has underflowed to zero")
+	if (epsilon == 0) 
+		stop("epsilon has underflowed to zero, because some entry and \n",
+				"exit times were too close together")
 	
 	# Define and possibly display stratumSizes and subcohortStratumSizes. 
 	stratumSizes <- tapply(stratum, stratum, length) 
@@ -259,14 +277,14 @@ cchs <- function(formula, data=sys.parent(), inSubcohort, stratum,
 		"\n## samplingFractions=", paste(sf,collapse=" "),	"\n\n", sep="")
 	}
 	
-	#############################################################################
+	############################################################################
 	# Store the column-names of modelMatrix, and replace the column-names by 
-	# dummy names so that they are different from all the extra variables' names. 
+	# dummy names so that they are different from all the extra variables' names 
 	modelMatrixColumnNames <- colnames(modelMatrix)
 	colnames(modelMatrix) <- paste0("dummyName", 1:ncol(modelMatrix))
 	
-	# Rename modelMatrix as modelMatrixPlus and make it be a data-frame that also 
-	# contains the "extra variables" such as inSubcohort and timeAtEntry. 
+	# Rename modelMatrix as modelMatrixPlus and make it be a data-frame that  
+	# also contains the "extra variables" such as inSubcohort and timeAtEntry. 
 	# See extra comments 3 and 4 below. 
 	modelMatrixPlus <- modelMatrix
 	rm(modelMatrix) 
@@ -279,7 +297,7 @@ cchs <- function(formula, data=sys.parent(), inSubcohort, stratum,
 
 	# Do the splitting (if swap=TRUE). 
 	if (swap) 
-		modelMatrixPlus <- doSplitting(modelMatrixPlus, epsilon, verbose=verbose)
+		modelMatrixPlus <- doSplitting(modelMatrixPlus, epsilon,verbose=verbose)
 
 	# Change the entry-time for the non-subcohort cases. 
 	nonSubcohortRows <- which(!modelMatrixPlus$inSubcohort)
@@ -291,8 +309,8 @@ cchs <- function(formula, data=sys.parent(), inSubcohort, stratum,
 			modelMatrixPlus$timeAtEntry != modelMatrixPlus$timeAtExit,] 
 	
 	# Create modelMatrixPlus$useForDfbeta. This indicates which rows of  
-	# modelMatrixPlus will be used in the calculations with the dfbeta residuals 
-	# to calculate the variances and CIs. 
+	# modelMatrixPlus will be used in the calculations with the dfbeta  
+	# residuals to calculate the variances and CIs. 
 	modelMatrixPlus$useForDfbeta <- modelMatrixPlus$inSubcohort
 	
 	# Make two adjustments if the user requested them. The point at which these 
@@ -307,11 +325,11 @@ cchs <- function(formula, data=sys.parent(), inSubcohort, stratum,
 		# (verbose=TRUE produces a huge amount of output)
 	
 	#######################################################################
-	# Avoid R CMD check problem (see http://stackoverflow.com/q/9439256/1310503):  
+	# Avoid R CMD check problem (http://stackoverflow.com/q/9439256/1310503):  
 	useForDfbeta <- NULL 
 
-	# Remake modelMatrix from modelMatrixPlus, by removing the extra columns, and
-	# make those be normal variables again. (After doing this, do not reorder or 
+	# Remake modelMatrix from modelMatrixPlus, by removing the extra columns, &
+	# make those be normal variables again. (After doing this, do not reorder or
 	# manipulate the rows of modelMatrix or the elements of inSubcohort etc.) 
 	# First, get the extra variables. 
 	varNames <- c(extraVariables, "useForDfbeta") 
@@ -327,26 +345,34 @@ cchs <- function(formula, data=sys.parent(), inSubcohort, stratum,
 	
 	# Fit the Cox model. coxph sometimes throws errors and warnings, so append 
 	# extra text to these to make it clear to the user of cchs where they come 
-	# from. (If coxph throws an error, append extra text and throw the error.
+	# from. 
+	# (Intercepting and annotating of errors and warnings, which is done if 
+	# annotateErrors=TRUE, works as follows. 
+	# If coxph throws an error, append extra text and throw the error.
 	# If it throws a warning, append extra text and throw the warning, but still
-	# store the result of coxph in result. This code for intercepting errors and
-	# warnings is based on http://stackoverflow.com/a/4952908/1310503.) 
-	result <- withCallingHandlers(
-		tryCatch(
-			coxph(Surv(timeAtEntry, timeAtExit, isCase) ~ modelMatrix + 
-				offset(-log(samplingFractions)), control=coxphControl),
-			error=function(e) stop(e$message, 
-					"\n[This error was thrown by coxph, which was called by cchs.]")
-		), 
-		warning=function(w) {
-			warning(conditionMessage(w), 
+	# store the result of coxph in result. This code for intercepting errors 
+	# and warnings is based on http://stackoverflow.com/a/4952908/1310503.) 
+	if (annotateErrors) {
+		result <- withCallingHandlers(
+			tryCatch(
+				coxph(Surv(timeAtEntry, timeAtExit, isCase) ~ modelMatrix + 
+					offset(-log(samplingFractions)), control=coxphControl),
+				error=function(e) stop(e$message, 
+				"\n[This error was thrown by coxph, which was called by cchs.]")
+			), 
+			warning=function(w) {
+				warning(conditionMessage(w), 
 				"\n[This warning was thrown by coxph, which was called by cchs.]")
-			invokeRestart("muffleWarning")
-		}
-	)
+				invokeRestart("muffleWarning")
+			}
+		)
+	} else {
+		result <- coxph(Surv(timeAtEntry, timeAtExit, isCase) ~ modelMatrix + 
+				offset(-log(samplingFractions)), control=coxphControl)
+	}
 	
 	# Note on the call to coxph: when a matrix/data-frame is used in the RHS of 
-	# a formula, this is taken to mean the columns separated by + (see ?formula). 
+	# a formula, this is taken to mean the columns separated by + (see ?formula) 
 
 	# Fix the names on some elements of result. (This needs to be done because 
 	# when a matrix is used in the RHS of a model formula, its name gets 
@@ -354,12 +380,12 @@ cchs <- function(formula, data=sys.parent(), inSubcohort, stratum,
 	names(result$coefficients) <- modelMatrixColumnNames
 	dimnames(result$var) <- list(modelMatrixColumnNames, modelMatrixColumnNames)
 	
-	#############################################################################
-	# Compute the variances. (This uses equation (5) in Langholz & Jiao 2007.)
+	############################################################################
+	# Calculate the variance matrix, using equation (5) in Langholz & Jiao 2007.
 
 	dfbetaUncombined <- residuals(result, type="dfbeta")
 	if (is.matrix(dfbetaUncombined)) {
-		dfbetaUncombined <- dfbetaUncombined[useForDfbeta,]
+		dfbetaUncombined <- dfbetaUncombined[useForDfbeta, , drop=FALSE]
 	} else {  # it must be a vector
 		dfbetaUncombined <- matrix(dfbetaUncombined[useForDfbeta], ncol=1)
 	} 
@@ -382,18 +408,17 @@ cchs <- function(formula, data=sys.parent(), inSubcohort, stratum,
 				var(dfbetaCombined[dfbetaStrata==i,])
 	}
 	
-	#############################################################################
+	############################################################################
 	# Create result$coeffsTable, which contains the hazard ratios, confidence 
 	# intervals, p-values, etc. This is largely based on code from 
 	# survival:::summary.coxph. See extra comment 5 below. 
 	beta <- result$coefficients
 	se <- sqrt(diag(result$var))
-	confidenceLevel <- 0.95
-	z <- qnorm((1 + confidenceLevel) / 2)  # z=1.96
+	z <- qnorm((1 + confidenceLevel) / 2)  # e.g. z=1.96
 	result$coeffsTable <- cbind(exp(beta), exp(beta-z*se), exp(beta+z*se), 
 			1-pchisq((beta/se)^2, 1), beta, se)		
 	colnames(result$coeffsTable) <- 
-			c("HR","CIlower","CIupper","p","logHR","SElogHR")
+			c("HR", "CIlower", "CIupper", "p", "logHR", "SElogHR")
 	# The p-value is for the Wald test. See Collett 1994/2003, "Modelling 
 	# Survival Data in Medical Research", second para of section 3.4, p67/69. 
 	
@@ -413,9 +438,13 @@ cchs <- function(formula, data=sys.parent(), inSubcohort, stratum,
 		message <- paste0(message, tiedTimesMessage)
 	result$message <- message
 		
+	# Store confidenceLevel, if it is not 0.95. 
+	if (confidenceLevel != 0.95) 
+		result$confidenceLevel <- confidenceLevel
+	
 	# If !keepAllCoxphElements, then drop the appropriate elements from result. 
 	varsToKeep <- c("coefficients","var","iter","n","nevent","nStrata","call",
-			"coeffsTable","nEachStatus","message")
+			"coeffsTable","nEachStatus","message","confidenceLevel")
 	if (!keepAllCoxphElements) 
 		for (varName in names(result))
 			if (!(varName %in% varsToKeep))
@@ -430,20 +459,29 @@ cchs <- function(formula, data=sys.parent(), inSubcohort, stratum,
 # Extra comments 
 
 # 1. modelFrame <- model.frame(...) is done twice, the first time to detect 
-# "rows" that are used in the model but contain NAs, the second time to create 
-# the model-frame that will be manipulated and passed to coxph. The reason for  
-# not doing it once to do both these things is as follows. The best way to find  
-# what formula-variables have NAs is to run model.frame with na.action=na.omit,  
-# and doing this also removes those rows from the resulting model-frame. (If  
-# you do model.frame with na.pass, then attr(modelFrame, "na.action") does not  
-# get created.) When making the model-frame that will be manipulated and passed  
-# to coxph, it is cleaner and safer to use "na.pass" (which means don't drop  
-# the NA-containing rows) and then remove all the NA-containing rows from  
-# modelMatrix, inSubcohort, etc. in one fell swoop. The NA-containing rows are  
-# identified by rowsWithNA. 
+# "rows" that would be used in the model but contain NAs, the second time to  
+# create the model-frame that will be manipulated and passed to coxph. The 
+# reason for not doing it once to do both these things is as follows. The best 
+# way to find what formula-variables have NAs is to run model.frame with 
+# na.action=na.omit, and doing this also removes those rows from the resulting 
+# model-frame. (If you do model.frame with na.pass, then attr(modelFrame, 
+# "na.action") does not get created.) When making the model-frame that will be  
+# manipulated and passed to coxph, it is cleaner and safer to use "na.pass" 
+# (which means don't drop the NA-containing rows) and then remove all the 
+# NA-containing rows from modelMatrix, inSubcohort, etc. in one fell swoop. 
+# The NA-containing rows are identified by rowsWithNA. 
+# 
+# There is an alternative way of dealing with NAs and which rows should be used.
+# Find which rows are dropped by coxph by including cluster(1:nrow(data)) in 
+# the formula and using model=TRUE, and then look at the "cluster(1:nrow(data))"
+# column of the coxph object. (It is then also necessary to use result$naive.var
+# instead of result$var when calculating the variance matrix.) 
+# However, creating modelFrame twice has various advantages: it is easier to 
+# catch errors in the Surv object or model-formula and easier to identify 
+# which elements of inSubcohort etc. need to be kept. 
 
 # 2. The assumption that all the levels of stratum appear in the data, i.e. that 
-# stratum has no unusedlevels like you get from as.factor(4:6)[-2], is made by:
+# stratum has no unused levels like you get from as.factor(4:6)[-2], is made by:
 # - the functions in samplingFractionSubroutines.R 
 # - the loop that finally calculates the correct value of result$var
 # - for (stra in levels(stratum)) ...
